/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */

#ifndef mozilla_dom_XPathNSResolverBinding_h
#define mozilla_dom_XPathNSResolverBinding_h

#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/BindingUtils.h"
#include "mozilla/dom/CallbackInterface.h"
#include "mozilla/dom/Nullable.h"

namespace mozilla {
namespace dom {

struct NativePropertyHooks;
class ProtoAndIfaceCache;
class XPathNSResolver;
struct XPathNSResolverAtoms;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

class XPathNSResolver : public CallbackInterface
{
public:
  explicit inline XPathNSResolver(JS::Handle<JSObject*> aCallback, nsIGlobalObject* aIncumbentGlobal)
    : CallbackInterface(aCallback, aIncumbentGlobal)
  {
  }

  template <typename T>
  inline void
  LookupNamespaceURI(const T& thisObjPtr, const nsAString& prefix, nsString& aRetVal, ErrorResult& aRv, ExceptionHandling aExceptionHandling = eReportExceptions, JSCompartment* aCompartment = nullptr)
  {
    CallSetup s(this, aRv, aExceptionHandling, aCompartment);
    if (!s.GetContext()) {
      aRv.Throw(NS_ERROR_UNEXPECTED);
      return;
    }
    JS::Rooted<JSObject*> thisObjJS(s.GetContext(),
      WrapCallThisObject(s.GetContext(), thisObjPtr));
    if (!thisObjJS) {
      aRv.Throw(NS_ERROR_FAILURE);
      return;
    }
    JS::Rooted<JS::Value> thisValJS(s.GetContext(),
                                    JS::ObjectValue(*thisObjJS));
    return LookupNamespaceURI(s.GetContext(), thisValJS, prefix, aRetVal, aRv);
  }

  inline void
  LookupNamespaceURI(const nsAString& prefix, nsString& aRetVal, ErrorResult& aRv, ExceptionHandling aExceptionHandling = eReportExceptions, JSCompartment* aCompartment = nullptr)
  {
    CallSetup s(this, aRv, aExceptionHandling, aCompartment);
    if (!s.GetContext()) {
      aRv.Throw(NS_ERROR_UNEXPECTED);
      return;
    }
    return LookupNamespaceURI(s.GetContext(), JS::UndefinedHandleValue, prefix, aRetVal, aRv);
  }

  inline bool
  operator==(const XPathNSResolver& aOther) const
  {
    return CallbackInterface::operator==(aOther);
  }

private:
  void LookupNamespaceURI(JSContext* cx, JS::Handle<JS::Value> aThisVal, const nsAString& prefix, nsString& aRetVal, ErrorResult& aRv);

  static bool
  InitIds(JSContext* cx, XPathNSResolverAtoms* atomsCache);
};


} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_XPathNSResolverBinding_h
