/* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

#ifndef GFX_SOFTWARE_VSYNC_SOURCE_H
#define GFX_SOFTWARE_VSYNC_SOURCE_H

#include "mozilla/Monitor.h"
#include "mozilla/RefPtr.h"
#include "mozilla/TimeStamp.h"
#include "base/thread.h"
#include "nsISupportsImpl.h"
#include "VsyncSource.h"

class CancelableTask;

class SoftwareDisplay MOZ_FINAL : public mozilla::gfx::VsyncSource::Display
{
  NS_INLINE_DECL_THREADSAFE_REFCOUNTING(SoftwareDisplay)

public:
  SoftwareDisplay();
  virtual void EnableVsync() MOZ_OVERRIDE;
  virtual void DisableVsync() MOZ_OVERRIDE;
  virtual bool IsVsyncEnabled() MOZ_OVERRIDE;
  bool IsInSoftwareVsyncThread();
  virtual void NotifyVsync(mozilla::TimeStamp aVsyncTimestamp) MOZ_OVERRIDE;
  void ScheduleNextVsync(mozilla::TimeStamp aVsyncTimestamp);

protected:
  ~SoftwareDisplay();

private:
  mozilla::TimeDuration mVsyncRate;
  // Use a chromium thread because nsITimers* fire on the main thread
  base::Thread* mVsyncThread;
  bool mVsyncEnabled;
  CancelableTask* mCurrentVsyncTask;
  // Locks against both mCurrentVsyncTask and mVsyncEnabled
  mozilla::Monitor mCurrentTaskMonitor;
}; // SoftwareDisplay

// Fallback option to use a software timer to mimic vsync. Useful for gtests
// To mimic a hardware vsync thread, we create a dedicated software timer
// vsync thread.
class SoftwareVsyncSource : public mozilla::gfx::VsyncSource
{
public:
  SoftwareVsyncSource();
  ~SoftwareVsyncSource();

  virtual Display& GetGlobalDisplay() MOZ_OVERRIDE
  {
    MOZ_ASSERT(mGlobalDisplay != nullptr);
    return *mGlobalDisplay;
  }

private:
  nsRefPtr<SoftwareDisplay> mGlobalDisplay;
};

#endif /* GFX_SOFTWARE_VSYNC_SOURCE_H */
