/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIVariant;

[uuid(256c9139-5a29-41e1-8698-f9f9aae7d6cf)]
interface nsIDOMFormData : nsISupports 
{
  void append(in DOMString name, in nsIVariant value);
};

%{ C++
#define NS_FORMDATA_CID \
{ 0x6b192618, 0x26f2, 0x426e, \
  { 0xa7, 0xac, 0x1e, 0x13, 0xa6, 0xa4, 0x52, 0x2b } }

#define NS_FORMDATA_CONTRACTID "@mozilla.org/files/formdata;1"
%}
